@@ -33,11 +33,11 @@ namespace ranges
         {
             template<typename Derived>
             using begin_adaptor_t =
-                detail::decay_t<decltype(range_access::begin_adaptor(std::declval<Derived &>(), 42))>;
+                detail::decay_t<decltype(range_access::begin_adaptor(std::declval<Derived &>()))>;
 
             template<typename Derived>
             using end_adaptor_t =
-                detail::decay_t<decltype(range_access::end_adaptor(std::declval<Derived &>(), 42))>;
+                detail::decay_t<decltype(range_access::end_adaptor(std::declval<Derived &>()))>;
 
             template<typename Derived>
             using adapted_iterator_t =
@@ -338,7 +338,7 @@ namespace ranges
                 return ranges::iter_move(first());
             }
             // If the adaptor does not have an iter_move function but overrides the read
-            // member function, apply std::move to the result of calling current.
+            // member function, apply std::move to the result of calling read.
             template<typename A = Adapt,
                 typename R = decltype(std::declval<A const &>().read(std::declval<BaseIter const &>())),
                 typename X = aux::move_t<R>>
@@ -405,9 +405,9 @@ namespace ranges
             static RANGES_CXX14_CONSTEXPR adaptor_cursor_t<D> begin_cursor_(D &d)
                 noexcept(noexcept(adaptor_cursor_t<D>{
                     std::declval<detail::begin_adaptor_t<D> &>().begin(d),
-                    range_access::begin_adaptor(d, 42)}))
+                    range_access::begin_adaptor(d)}))
             {
-                auto adapt = range_access::begin_adaptor(d, 42);
+                auto adapt = range_access::begin_adaptor(d);
                 auto pos = adapt.begin(d);
                 return {std::move(pos), std::move(adapt)};
             }
@@ -429,9 +429,9 @@ namespace ranges
             static RANGES_CXX14_CONSTEXPR adaptor_sentinel_t<D> end_cursor_(D &d)
                 noexcept(noexcept(adaptor_sentinel_t<D>{
                     std::declval<detail::end_adaptor_t<D> &>().end(d),
-                    range_access::end_adaptor(d, 42)}))
+                    range_access::end_adaptor(d)}))
             {
-                auto adapt = range_access::end_adaptor(d, 42);
+                auto adapt = range_access::end_adaptor(d);
                 auto pos = adapt.end(d);
                 return {std::move(pos), std::move(adapt)};
             }
