@@ -27,6 +27,9 @@
 #include "./simple_test.hpp"
 #include "./test_iterators.hpp"
 
+RANGES_DIAGNOSTIC_PUSH
+RANGES_DIAGNOSTIC_IGNORE_DEPRECATED_THIS_CAPTURE
+
 struct check_equal_fn
 {
     template<typename T, typename U>
@@ -61,7 +64,10 @@ struct check_equal_fn
     }
 };
 
-RANGES_INLINE_VARIABLE(check_equal_fn, check_equal)
+inline namespace function_objects
+{
+    RANGES_INLINE_VARIABLE(check_equal_fn, check_equal)
+}
 
 template<typename Expected, typename Actual>
 void has_type(Actual &&)
@@ -104,8 +110,6 @@ private:
     }
 public:
     function_ref() = default;
-    function_ref(function_ref &) = default;
-    function_ref(function_ref const &) = default;
     template<typename T>
     function_ref(T const &t)
       : data_(&t)
@@ -297,4 +301,6 @@ struct MoveOnlyString
     }
 };
 
+RANGES_DIAGNOSTIC_POP
+
 #endif
