@@ -9,9 +9,11 @@
 //
 // Project home: https://github.com/ericniebler/range-v3
 
+#include <map>
 #include <vector>
 #include <range/v3/core.hpp>
 #include <range/v3/view/iota.hpp>
+#include <range/v3/view/map.hpp>
 #include <range/v3/view/take.hpp>
 #include <range/v3/view/reverse.hpp>
 #include <range/v3/view/any_view.hpp>
@@ -109,6 +111,13 @@ RANGES_DIAGNOSTIC_POP
         static_assert((get_categories<decltype(ints)>() & category::random_access) == category::random_access, "");
         static_assert((get_categories<decltype(ints)>() & category::sized) == category::sized, "");
     }
+    {
+        any_view<int, category::input | category::sized> ints = view::ints | view::take_exactly(10);
+        CONCEPT_ASSERT(InputView<decltype(ints)>());
+        CONCEPT_ASSERT(SizedView<decltype(ints)>());
+        static_assert((get_categories<decltype(ints)>() & category::input) == category::input, "");
+        static_assert((get_categories<decltype(ints)>() & category::sized) == category::sized, "");
+    }
     {
         any_view<int, category::bidirectional> ints = view::ints;
         CONCEPT_ASSERT(BidirectionalView<decltype(ints)>());
@@ -165,6 +174,14 @@ RANGES_DIAGNOSTIC_POP
         ::check_equal(v, ten_ints);
     }
 
+    // Regression test for #880
+    {
+        std::map<int, int> mm{ {0, 1}, {2, 3} };
+        ranges::any_view<int, ranges::category::forward | ranges::category::sized> as_any =
+            mm | ranges::view::keys;
+        (void)as_any;
+    }
+
     test_polymorphic_downcast();
 
     return test_result();
