@@ -16,6 +16,7 @@
 #include <range/v3/core.hpp>
 #include <range/v3/view/chunk.hpp>
 #include <range/v3/view/cycle.hpp>
+#include <range/v3/view/filter.hpp>
 #include <range/v3/view/iota.hpp>
 #include <range/v3/view/join.hpp>
 #include <range/v3/view/move.hpp>
@@ -116,7 +117,7 @@ int main()
         ::models<concepts::RandomAccessRange>(rng);
         auto it = rng.begin();
         auto it2 = next(it,3);
-        CHECK((it2 - it) == 0);
+        CHECK((it2 - it) == 3);
         ::check_equal(*it, {5,5,5});
         ::check_equal(*it2, {5,5,5});
     }
@@ -132,15 +133,14 @@ int main()
         auto it2 = next(it,2);
         ::check_equal(*it, {0,1});
         ::check_equal(*it2, {1,2});
-        // Strange, but not wrong necessarily:
         CHECK((it - it) == 0);
         CHECK((next(it,1) - it) == 1);
-        // static_cast<void>(next(it,2) - it); // not in the domain of -
-        CHECK((next(it,3) - it) == 0);
-        CHECK((next(it,4) - it) == 1);
-        // static_cast<void>(next(it,5) - it); // not in the domain of -
-        CHECK((next(it,6) - it) == 0);
-        CHECK((next(it,7) - it) == 1);
+        CHECK((next(it,2) - it) == 2);
+        CHECK((next(it,3) - it) == 3);
+        CHECK((next(it,4) - it) == 4);
+        CHECK((next(it,5) - it) == 5);
+        CHECK((next(it,6) - it) == 6);
+        CHECK((next(it,7) - it) == 7    );
     }
 
     {
@@ -156,13 +156,13 @@ int main()
         ::check_equal(*it2, {2,0,1,2});
         // Strange, but not wrong necessarily:
         CHECK((it - it) == 0);
-        // static_cast<void>(next(it,1) - it); // not in the domain of -
-        // static_cast<void>(next(it,2) - it); // not in the domain of -
-        CHECK((next(it,3) - it) == 0);
-        // static_cast<void>(next(it,4) - it); // not in the domain of -
-        // static_cast<void>(next(it,5) - it); // not in the domain of -
-        CHECK((next(it,6) - it) == 0);
-        // static_cast<void>(next(it,7) - it); // not in the domain of -
+        CHECK((next(it,1) - it) == 1);
+        CHECK((next(it,2) - it) == 2);
+        CHECK((next(it,3) - it) == 3);
+        CHECK((next(it,4) - it) == 4);
+        CHECK((next(it,5) - it) == 5);
+        CHECK((next(it,6) - it) == 6);
+        CHECK((next(it,7) - it) == 7);
     }
 
     {
@@ -181,15 +181,16 @@ int main()
         CHECK((next(it,1) - it) == 1);
         CHECK((next(it,2) - it) == 2);
         CHECK((next(it,3) - it) == 3);
-        // static_cast<void>(next(it,4) - it); // not in the domain of -
-        // static_cast<void>(next(it,5) - it); // not in the domain of -
-        // ...
-        // static_cast<void>(next(it,8) - it); // not in the domain of -
-        // static_cast<void>(next(it,9) - it); // not in the domain of -
-        CHECK((next(it,10) - it) == 0);
-        CHECK((next(it,11) - it) == 1);
-        CHECK((next(it,12) - it) == 2);
-        CHECK((next(it,13) - it) == 3);
+        CHECK((next(it,4) - it) == 4);
+        CHECK((next(it,5) - it) == 5);
+        CHECK((next(it,6) - it) == 6);
+        CHECK((next(it,7) - it) == 7);
+        CHECK((next(it,8) - it) == 8);
+        CHECK((next(it,9) - it) == 9);
+        CHECK((next(it,10) - it) == 10);
+        CHECK((next(it,11) - it) == 11);
+        CHECK((next(it,12) - it) == 12);
+        CHECK((next(it,13) - it) == 13);
     }
 
     test_input_ranges();
@@ -212,5 +213,21 @@ int main()
         ::check_equal(rng | view::chunk(2), expected);
     }
 
+    {
+        // Regression test for https://stackoverflow.com/questions/49210190
+        auto rng = view::closed_iota(1,25)
+            | view::filter([](int item){ return item % 10 != 0; })
+            | view::chunk(10);
+        auto it = ranges::begin(rng);
+        auto last = ranges::end(rng);
+        CHECK(it != last);
+        ::check_equal(*it, {1,2,3,4,5,6,7,8,9,11});
+        CHECK(++it != last);
+        ::check_equal(*it, {12,13,14,15,16,17,18,19,21,22});
+        CHECK(++it != last);
+        ::check_equal(*it, {23,24,25});
+        CHECK(++it == last);
+    }
+
     return ::test_result();
 }
