@@ -13,9 +13,10 @@
 #include <cctype>
 #include <range/v3/core.hpp>
 #include <range/v3/view/counted.hpp>
-#include <range/v3/view/split.hpp>
+#include <range/v3/view/c_str.hpp>
 #include <range/v3/view/empty.hpp>
 #include <range/v3/view/remove_if.hpp>
+#include <range/v3/view/split.hpp>
 #include "../simple_test.hpp"
 #include "../test_utils.hpp"
 #include "../test_iterators.hpp"
@@ -155,5 +156,11 @@ int main()
         }
     }
 
+    {   // Regression test for https://stackoverflow.com/questions/49015671
+        auto const str = "quick brown fox";
+        auto rng = view::c_str(str) | view::split(' ');
+        CONCEPT_ASSERT(ForwardRange<decltype(rng)>());
+    }
+
     return test_result();
 }
