@@ -17,6 +17,7 @@
 #include <range/v3/view/move.hpp>
 #include <range/v3/view/reverse.hpp>
 #include <range/v3/view/stride.hpp>
+#include <range/v3/view/iota.hpp>
 #include <range/v3/algorithm/copy.hpp>
 #include <range/v3/utility/iterator.hpp>
 #include <range/v3/numeric.hpp>
@@ -115,5 +116,65 @@ int main()
         ::check_equal(rng, {0,2,4,6});
     }
 
-    return ::test_result();
+    {
+        std::list<int> li;
+        copy(v, back_inserter(li));
+        iterator_range<std::list<int>::const_iterator> tmp{li.begin(), li.end()};
+        auto rng = tmp | view::stride(3);
+        using R = decltype(rng);
+        CONCEPT_ASSERT(BidirectionalView<R>());
+        CONCEPT_ASSERT(!RandomAccessRange<R>());
+        CONCEPT_ASSERT(!BoundedRange<R>());
+        CONCEPT_ASSERT(!SizedRange<R>());
+        CONCEPT_ASSERT(!Range<R const>());
+        ::check_equal(rng,
+                    {0, 3, 6, 9, 12, 15, 18, 21, 24, 27, 30, 33, 36, 39, 42, 45, 48});
+        ::check_equal(rng | view::reverse,
+                    {48, 45, 42, 39, 36, 33, 30, 27, 24, 21, 18, 15, 12, 9, 6, 3, 0});
+    }
+
+    {
+        std::list<int> li;
+        copy(v, back_inserter(li));
+        sized_iterator_range<std::list<int>::const_iterator> tmp{li.begin(), li.end(), li.size()};
+        auto rng = tmp | view::stride(3);
+        using R = decltype(rng);
+        CONCEPT_ASSERT(BidirectionalView<R>());
+        CONCEPT_ASSERT(!RandomAccessRange<R>());
+        CONCEPT_ASSERT(BoundedRange<R>());
+        CONCEPT_ASSERT(SizedRange<R>());
+        CONCEPT_ASSERT(Range<R const>());
+        CHECK((*--rng.end()) == 48);
+        ::check_equal(rng,
+                    {0, 3, 6, 9, 12, 15, 18, 21, 24, 27, 30, 33, 36, 39, 42, 45, 48});
+        ::check_equal(rng | view::reverse,
+                    {48, 45, 42, 39, 36, 33, 30, 27, 24, 21, 18, 15, 12, 9, 6, 3, 0});
+    }
+
+    // https://github.com/ericniebler/range-v3/issues/901
+    {
+        auto r = view::iota( 0, 12 );
+
+        // Evenly divisible stride:
+        auto strided1 = r | view::stride(3);
+        ::check_equal(strided1, {0, 3, 6, 9});
+        CHECK(strided1.size() == 4u);
+        CHECK(strided1.front() == 0);
+        CHECK(strided1[0] == 0);
+        CHECK(strided1.back() == 9);
+        CHECK(strided1[3] == 9);
+        CHECK(strided1[(int)strided1.size() - 1] == 9);
+
+        // Not evenly divisible stride:
+        auto strided2 = r | view::stride(5);
+        ::check_equal(strided2, {0, 5, 10});
+        CHECK(strided2.size() == 3u);
+        CHECK(strided2.front() == 0);
+        CHECK(strided2[0] == 0);
+        CHECK(strided2.back() == 10);
+        CHECK(strided2[2] == 10);
+        CHECK(strided2[(int)strided2.size() - 1] == 10);
+    }
+
+    //return ::test_result();
 }
